/*
 * Created on Nov 1, 2004
 *
 */
package com.sunlabs.archsim.engine;

import com.sunlabs.archsim.base.simulation.Event;





/**
 *
 * Copyright  2005 Sun Microsystems Inc. All rights reserved. Use is subject to license terms.
 * 
 * EventQueue is a priority queue
 * The implementation of the queue is a heap
 * @author ib27688
 */
public class EventHeapQueue extends EventQueue {
	
	protected EventHeap myHeap;
	
	/**
	 * Make an event queue of the default initial capacity
	 */
	public EventHeapQueue() {
		myHeap= new EventHeap();
	} // event queue constructor
	
	/**
	 * Make an event queue with the specified capacity
	 * @param capacity Initial queue capacity
	 */
	public EventHeapQueue(int capacity) {
		myHeap= new EventHeap(capacity);
	} // EventHeap(int)
	
	/**
	 * Obtain current queue capacity
	 * @return current queue capacity
	 */
	public int capacity() {
		return myHeap.capacity();
	} // capacity
	
	/** 
	 * Remove the next event from the queue and pass it on for execution
	 * @return the next event from the queue
	 */
	public Event nextEvent() {
		return myHeap.getTop();
	} // getNextEvent
	
	/**
	 * Peek at the top of the queue, do not remove the next event
	 * @return The next event to be executed
	 */
	public Event peek() {
		return myHeap.peek();
	} // peek
	
	/**
	 * Insert a new event in the queue
	 * @param newEvent The new event to be inserted to the queue
	 */
	public void insertEvent(Event newEvent) {
		myHeap.insert(newEvent);
	} // insert
	
	/**
	 * 
	 * @return the size of the queue
	 */
	public int size() {
		return myHeap.size();
	} // size
	
	/**
	 * Is the queue empty 
	 * @return the boolean indicating whether the queue is empty
	 */
	public boolean isEmpty() {
		return (0 == size());
	} // is Empty
	
	/**
	 * clear the queue
	 *
	 */
	public void clear() {
		myHeap.clear();
	} // clear
	
	public void print() {
		System.out.println("Queue:");
		myHeap.print();
		System.out.println();
	}
} // class EventQueue
